@@ -83,8 +83,7 @@ def __try_import__(modulename):
         return False
 
 def _version_to_string(version):
-    import string
-    return string.join([str(x) for x in version], ".")
+    return ".".join([str(x) for x in version])
 
 def _string_to_list(version):
     return [int(x) for x in version.split(".")]
@@ -141,6 +140,9 @@ def _checks():
     if not __try_import__("gtk.glade"):
         return (_("Could not import the Python bindings for libglade"),
                 _("Make sure you have the Python bindings for libglade installed"))
+    if not __try_import__("goocanvas"):
+        return (_("Could not import the Python bindings for goocanvas"),
+                _("Make sure you have the Python bindings for goocanvas installed"))
     req, inst = check_required_version("gtk")
     if req:
         return (_("You do not have a recent enough version of the GTK+ Python Bindings (currently %s)") % inst,
