@@ -19,6 +19,10 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
+"""
+Plugin Manager
+"""
+
 import os
 import os.path
 import stat
@@ -31,12 +35,12 @@ except:
     import pickle
 import pkg_resources
 import gtk
-import gobject
-import zope.interface.verify 
+import zope.interface.verify
 
+from signalinterface import Signallable
 import plugincore
 
-class PluginManager(gobject.GObject):
+class PluginManager(object, Signallable):
     """
     Manages plugins in a centralized way.
 
@@ -59,36 +63,29 @@ class PluginManager(gobject.GObject):
 
     """
 
-    __gsignals__ = {
-        "plugin-enabled-changed" : (gobject.SIGNAL_RUN_LAST,\
-                                      gobject.TYPE_NONE,\
-                                      (gobject.TYPE_PYOBJECT,)),
-        "plugin-installed" : (gobject.SIGNAL_RUN_LAST,\
-                                      gobject.TYPE_NONE,\
-                                      (gobject.TYPE_PYOBJECT,)),
-        "plugin-uninstalled" : (gobject.SIGNAL_RUN_LAST,\
-                                      gobject.TYPE_NONE,\
-                                      (gobject.TYPE_PYOBJECT,)),
+    __signals__ = {
+        "plugin-enabled-changed" : ["plugin_names"],
+        "plugin-installed" : ["plugin_name"],
+        "plugin-uninstalled" : ["plugin_name"],
 
         }
 
     def __init__(self, local_plugin_path, settings_path):
-        """ 
-        Initialize a new plugin manager 
+        """
+        Initialize a new plugin manager
 
         @param local_plugin_path: local path where new plugins will be installed
         @param settings_path: path where all plugin settings are stored
         """
-        gobject.GObject.__init__(self)
 
         # plugins are collected in a bag (a structure relatng 1->many)
         # {"my_plugin":{"plugin":plugin_object, "filename":"/home/luca/plugins/my_plugin.py"}}
-        self.pluginbag={}
+        self.pluginbag = {}
 
         # Store plugin repositiories in the instance ensuring they are expressed
         # as absolute paths.
         self.local_plugin_path = os.path.abspath(local_plugin_path)
-        self.plugin_paths = [self.local_plugin_path,]
+        self.plugin_paths = [ self.local_plugin_path, ]
         self.settings_path = os.path.abspath(settings_path)
         self.collect()
 
@@ -127,8 +124,7 @@ class PluginManager(gobject.GObject):
 
         plugin_filename = self.pluginbag[plugin.name]["filename"]
 
-        #split plugin filename in dirname + basename
-        plugin_dirname = os.path.dirname(plugin_filename)
+        #Extract plugin base_name
         plugin_basename = os.path.basename(plugin_filename)
 
         #strip file extension from plugin basename ("myplugin.py" -> "myplugin")
@@ -138,8 +134,8 @@ class PluginManager(gobject.GObject):
         return os.path.join(self.settings_path, plugin_basename + ".conf")
 
     def _get_settings(self, plugin):
-        """ 
-        Get the plugin settings dictionary from disk 
+        """
+        Get the plugin settings dictionary from disk
 
         @param plugin: plugin for which to retrieve settings from disk
         @return: settings dictionary, None if not found
@@ -160,7 +156,7 @@ class PluginManager(gobject.GObject):
 
     def _is_valid_plugin_class(self, plugin_class):
         """
-        Check if plugin class correctly implements the IPlugin interface 
+        Check if plugin class correctly implements the IPlugin interface
 
         @param plugin_class: class to validate
         @return: True if class is a valid plugin, False otherwise
@@ -223,10 +219,10 @@ class PluginManager(gobject.GObject):
 
         elif filename.endswith(".egg"):
             # file is an egg, ask its entry point for the plugin class
-            fullName = os.path.join(directory, filename)
+            fullname = os.path.join(directory, filename)
 
-            pkg_resources.working_set.add_entry(fullName)
-            dist_generator = pkg_resources.find_distributions(fullName)
+            pkg_resources.working_set.add_entry(fullname)
+            dist_generator = pkg_resources.find_distributions(fullname)
             for dist in dist_generator:
                 try:
                     plugin_class = dist.load_entry_point("pitivi.plugins", "plugin")
@@ -330,7 +326,7 @@ class PluginManager(gobject.GObject):
         if not (filename.endswith(".egg") or filename.endswith(".py")):
             print "File must be a .py or a .egg"
 
-        plugin_class = self.loadPluginClass(os.path.basename(filename),\
+        plugin_class = self.loadPluginClass(os.path.basename(filename), \
                                             os.path.dirname(filename))
 
         if not plugin_class:
@@ -342,7 +338,7 @@ class PluginManager(gobject.GObject):
             shutil.copy(filename, repository_path)
 
             # insert the new plugin filename entry
-            new_filename = os.path.join(os.path.abspath(repository_path),\
+            new_filename = os.path.join(os.path.abspath(repository_path), \
                                         os.path.basename(filename))
             self.pluginbag[plugin_class.name] = {\
                 "plugin": None, "filename": new_filename}
@@ -369,7 +365,7 @@ class PluginManager(gobject.GObject):
         @return: plugin if installed, None otherwise
         """
 
-        new_plugin_class = self.loadPluginClass(os.path.basename(filename),\
+        new_plugin_class = self.loadPluginClass(os.path.basename(filename), \
                                                 os.path.dirname(filename))
         old_plugin = self.alreadyLoaded(new_plugin_class)
         old_settings = self._get_settings(old_plugin)
