@@ -25,14 +25,15 @@ Timeline objects
 
 import weakref
 from random import randint
-import gobject
 import gst
 from pitivi.serializable import Serializable
 from pitivi.objectfactory import ObjectFactory
+from pitivi.signalinterface import Signallable
+import pitivi.instance as instance
 
-MEDIA_TYPE_NONE = 0
-MEDIA_TYPE_AUDIO = 1
-MEDIA_TYPE_VIDEO = 2
+(MEDIA_TYPE_NONE,
+ MEDIA_TYPE_AUDIO,
+ MEDIA_TYPE_VIDEO) = range(3)
 
 ## * Object Hierarchy
 
@@ -54,10 +55,20 @@ MEDIA_TYPE_VIDEO = 2
 ##         |
 ##         +---- Complex Effect (N->1)
 
-class BrotherObjects(gobject.GObject, Serializable):
+class BrotherObjects(Serializable, Signallable):
     """
     Base class for objects that can have a brother and be linked to something else
 
+    Properties:
+      _ Linked Object
+        _ Can be None
+        _ Must have same duration
+      _ Brother object
+        _ This is the same object but with the other media_type
+
+    Signals:
+      _ 'linked-changed' : new linked object
+
     Save/Load properties:
     * (optional) 'linked' (int) : UID of linked object
     * (optional) 'brother' (int) : UID of brother object
@@ -65,10 +76,8 @@ class BrotherObjects(gobject.GObject, Serializable):
 
     __data_type__ = "timeline-brother-objects"
 
-    __gsignals__ = {
-        "linked-changed" : (gobject.SIGNAL_RUN_LAST,
-                            gobject.TYPE_NONE,
-                            (gobject.TYPE_PYOBJECT, ))
+    __signals__ = {
+        "linked-changed" : ["brother"]
         }
 
     # UID (int) => object (BrotherObjects) mapping.
@@ -78,33 +87,43 @@ class BrotherObjects(gobject.GObject, Serializable):
     # pending UID (int) => objects (list of BrotherObjects and extra field)
     __waiting_for_pending_objects__ = {}
 
+    __deadband = 0
+    __edges = []
+
     def __init__(self, **unused_kw):
-        gobject.GObject.__init__(self)
-        self.linked = None
-        self.brother = None
+        self._linked = None
+        self._brother = None
         self.uid = -1
 
-    def _unlinkObject(self):
-        # really unlink the objects
-        if self.linked:
-            self.linked = None
-            self.emit("linked-changed", None)
+    ## properties
 
-    def _linkObject(self, object):
-        # really do the link
-        self.linked = object
-        self.emit("linked-changed", self.linked)
+    def _get_brother(self):
+        return self.getBrother()
 
-    def linkObject(self, object):
+    def _set_brother(self, brother):
+        self.setBrother(brother)
+    brother = property(_get_brother, _set_brother,
+                       doc="Brother object")
+
+    ## read-only properties
+
+    @property
+    def linked(self):
+        """ Linked object """
+        return self._linked
+
+
+    ## public API
+
+    def linkObject(self, obj):
         """
         link another object to this one.
         If there already is a linked object ,it will unlink it
         """
-        if self.linked and not self.linked == object:
+        if self._linked and not self._linked == obj:
             self.unlinkObject()
-        self._linkObject(object)
-        self.linked._linkObject(self)
-        pass
+        self._linkObject(obj)
+        self._linked._linkObject(self)
 
     def getLinkedObject(self):
         """
@@ -117,7 +136,8 @@ class BrotherObjects(gobject.GObject, Serializable):
         """
         unlink from the current linked object
         """
-        self.linked._unlinkObject()
+        if self._linked:
+            self._linked._unlinkObject()
         self._unlinkObject()
 
     def relinkBrother(self):
@@ -125,40 +145,57 @@ class BrotherObjects(gobject.GObject, Serializable):
         links the object back to it's brother
         """
         # if already linked, unlink from previous
-        if self.linked:
+        if self._linked:
             self.unlinkObject()
 
         # link to brother
-        if self.brother:
-            self.linkObject(self.brother)
+        if self._brother:
+            self.linkObject(self._brother)
 
     def getBrother(self, autolink=True):
         """
         returns the brother element if it's possible,
         if autolink, then automatically link it to this element
         """
-        if not self.brother:
-            self.brother = self._makeBrother()
-            if not self.brother:
+        if not self._brother:
+            self._brother = self._makeBrother()
+            if not self._brother:
                 return None
-        if autolink and not self.linked == self.brother:
+        if autolink and not self._linked == self._brother:
             self.relinkBrother()
-        return self.brother
+        return self._brother
 
     def setBrother(self, brother, autolink=True):
         """
         Force a brother on an object.
         This can be useful if it's the parent of the object that knows
         what his brother is.
-        Use with caution
+
+        Use with caution !!!
         """
         gst.log("brother:%r , autolink:%r" % (brother, autolink))
-        self.brother = brother
-        # set ourselves as our brother's brother
-        self.brother.brother = self
+        self._brother = brother
+        if self._brother:
+            # set ourselves as our brother's brother
+            self._brother._brother = self
         if autolink:
             self.relinkBrother()
 
+    # private methods
+
+    def _unlinkObject(self):
+        # really unlink the objects
+        if self._linked:
+            self._linked = None
+            self.emit("linked-changed", None)
+
+    def _linkObject(self, obj):
+        # really do the link
+        self._linked = obj
+        self.emit("linked-changed", self._linked)
+
+    # methods to override in subclasses
+
     def _makeBrother(self):
         """
         Make the exact same object for the other media_type
@@ -171,10 +208,10 @@ class BrotherObjects(gobject.GObject, Serializable):
     def toDataFormat(self):
         ret = Serializable.toDataFormat(self)
         ret["uid"] = self.getUniqueID()
-        if self.brother:
-            ret["brother-uid"] = self.brother.getUniqueID()
-        if self.linked:
-            ret["linked-uid"] = self.linked.getUniqueID()
+        if self._brother:
+            ret["brother-uid"] = self._brother.getUniqueID()
+        if self._linked:
+            ret["linked-uid"] = self._linked.getUniqueID()
         return ret
 
     def fromDataFormat(self, obj):
@@ -273,7 +310,8 @@ class BrotherObjects(gobject.GObject, Serializable):
             cls.__waiting_for_pending_objects__[uid] = []
         cls.__waiting_for_pending_objects__[uid].append((weakref.proxy(obj), extra))
 
-gobject.type_register(BrotherObjects)
+
+
 
 class TimelineObject(BrotherObjects):
     """
@@ -283,15 +321,9 @@ class TimelineObject(BrotherObjects):
       _ Start/Duration Time
       _ Media Type
       _ Gnonlin Object
-      _ Linked Object
-        _ Can be None
-        _ Must have same duration
-      _ Brother object
-        _ This is the same object but with the other media_type
 
     * signals
       _ 'start-duration-changed' : start position, duration position
-      _ 'linked-changed' : new linked object
 
     Save/Load properties
     * 'start' (int) : start position in nanoseconds
@@ -307,57 +339,156 @@ class TimelineObject(BrotherObjects):
     # order to create their gnlobject.
     __requires_factory__ = True
 
-    __gsignals__ = {
-        "start-duration-changed" : ( gobject.SIGNAL_RUN_LAST,
-                                 gobject.TYPE_NONE,
-                                 (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT, ))
+    __signals__ = {
+        "start-duration-changed" : ["start", "duration"],
+        "selected-changed" : [],
         }
 
+    __editable_properties__ = (
+        # (str: name, type: type, bool: is_controllable)
+    )
+
     def __init__(self, factory=None, start=gst.CLOCK_TIME_NONE,
                  duration=0, media_type=MEDIA_TYPE_NONE, name="", **kwargs):
         BrotherObjects.__init__(self, **kwargs)
         self.name = name
         gst.log("new TimelineObject :%s %r" % (name, self))
-        self.start = start
-        self.duration = duration
-        self.factory = None
+        self._start = start
+        if duration == 0 and factory:
+            duration = factory.default_duration
+        self._duration = duration
+        self._factory = None
         # Set factory and media_type and then create the gnlobject
         self.media_type = media_type
         self.gnlobject = None
+        self.factory = factory
+
+    ## properties
+
+    def _get_start(self):
+        return self._start
+
+    def _set_start(self, start):
+        self.setStartDurationTime(start=start)
+    start = property(_get_start, _set_start,
+                     doc="Start position of the object in its container (in nanoseconds)")
+
+    def _get_duration(self):
+        return self._duration
+
+    def _set_duration(self, duration):
+        self.setStartDurationTime(duration=duration)
+    duration = property(_get_duration, _set_duration,
+                        doc="Duration of the object in its container (in nanoseconds)")
+
+    def _get_factory(self):
+        return self._factory
+
+    def _set_factory(self, factory):
         self._setFactory(factory)
+    factory = property(_get_factory, _set_factory,
+                       doc="ObjectFactory used for this object")
+
+    def __get_selected(self):
+        return self._selected
+
+    def __set_selected(self, value):
+        self._selected = value
+        self.emit("selected-changed")
+    selected = property(__get_selected, __set_selected)
+
+    ## read-only properties
+
+    @property
+    def isaudio(self):
+        """ Boolean indicating whether the object produces Audio """
+        return self.media_type == MEDIA_TYPE_AUDIO
+
+    @property
+    def isvideo(self):
+        """ Boolean indicating whether the object produces Video """
+        return self.media_type == MEDIA_TYPE_VIDEO
+
+    ## public API
+
+    def setStartDurationTime(self, start=gst.CLOCK_TIME_NONE, duration=0):
+        """
+        Sets the start and/or duration time
+
+        Only use this method when you wish to modify BOTH start and duration at once
+        """
+        self._setStartDurationTime(start, duration)
+        if self._linked:
+            self._linked._setStartDurationTime(start, duration)
+
+    def snapStartDurationTime(self, start=gst.CLOCK_TIME_NONE, duration=0):
+        """ sets the start and/or duration time, with edge snapping """
+        self.setStartDurationTime(
+            instance.PiTiVi.current.timeline.snapObjToEdge(self, start),
+            duration)
+
+    def setInTime(self, time):
+        """Sets the timeline object's in point in the timeline, keeping its
+        out-point constant."""
+        delta = self.start - time 
+        self.setStartDurationTime(time, self.duration + delta)
+
+    def setOutTime(self, time):
+        """Set's the timeline object's out point in the timeline, keeping its
+        in-point constant."""
+        self.setStartDurationTime(self.start, time - self.start)
+
+    def snapInTime(self, time):
+        self.setInTime(instance.PiTiVi.current.timeline.snapTimeToEdge(time))
+
+    def snapOutTime(self, time):
+        self.setOutTime(instance.PiTiVi.current.timeline.snapTimeToEdge(time))
+
+    ## methods to override in subclasses
+
+    def _makeGnlObject(self):
+        """ create and return the gnl_object """
+        raise NotImplementedError
+
+## private methods
 
     def __repr__(self):
         if hasattr(self, "name"):
             return "<%s '%s' at 0x%x>" % (type(self).__name__, self.name, id(self))
         return "<%s at 0x%x>" % (type(self).__name__, id(self))
 
-    def _makeGnlObject(self):
-        """ create and return the gnl_object """
-        raise NotImplementedError
-
     def _setFactory(self, factory):
-        if self.factory:
-            gst.warning("Can't set a factory, this object already has one : %r" % self.factory)
+        if self._factory:
+            gst.warning("Can't set a factory, this object already has one : %r" % self._factory)
             return
+        if factory !=None and not isinstance(factory, ObjectFactory):
+            raise TypeError, "factory provided is not an ObjectFactory"
         gst.log("factory:%r requires factory:%r" % (factory, self.__requires_factory__))
-        self.factory = factory
-        if not self.__requires_factory__ or self.factory:
+        self._factory = factory
+        if not self.__requires_factory__ or self._factory:
             gst.log("%r Creating associated gnlobject" % self)
-            self.gnlobject = self._makeGnlObject()
-            self.gnlobject.connect("notify::start", self._startDurationChangedCb)
+            tmpgnl = self._makeGnlObject()
+            if tmpgnl == None:
+                raise Exception("We didn't get gnlobject for %r" % self)
+            self.gnlobject = tmpgnl
             self.gnlobject.log("got gnlobject !")
+            self.gnlobject.connect("notify::start", self._startDurationChangedCb)
             self.gnlobject.connect("notify::duration", self._startDurationChangedCb)
-            self._setStartDurationTime(self.start, self.duration, True)
+            self._setStartDurationTime(self._start, self._duration, force=True)
 
     def _setStartDurationTime(self, start=gst.CLOCK_TIME_NONE, duration=0, force=False):
         # really modify the start/duration time
         self.gnlobject.info("start:%s , duration:%s" %( gst.TIME_ARGS(start),
                                                         gst.TIME_ARGS(duration)))
         if duration > 0 and (not self.duration == duration or force):
-            self.duration = duration
+            duration = max(duration, 0)
+            if self.factory:
+                duration = min(duration, self.factory.duration)
+            self._duration = duration
             self.gnlobject.set_property("duration", long(duration))
         if not start == gst.CLOCK_TIME_NONE and (not self.start == start or force):
-            self.start = start
+            start = max(0, start)
+            self._start = start
             self.gnlobject.set_property("start", long(start))
 
     def setStartDurationTime(self, start=gst.CLOCK_TIME_NONE, duration=0):
@@ -366,50 +497,73 @@ class TimelineObject(BrotherObjects):
         if self.linked:
             self.linked._setStartDurationTime(start, duration)
 
+    def snapStartDurationTime(self, start=gst.CLOCK_TIME_NONE, duration=0):
+        """ sets the start and/or duration time, with edge snapping """
+        self.setStartDurationTime(
+            instance.PiTiVi.current.timeline.snapObjToEdge(self, start),
+            duration)
+
+    def setInTime(self, time):
+        """Sets the timeline object's in point in the timeline, keeping its
+        out-point constant."""
+        delta = self.start - time 
+        self.setStartDurationTime(time, self.duration + delta)
+
+    def setOutTime(self, time):
+        """Set's the timeline object's out point in the timeline, keeping its
+        in-point constant."""
+        self.setStartDurationTime(self.start, time - self.start)
+
+    def snapInTime(self, time):
+        self.setInTime(instance.PiTiVi.current.timeline.snapTimeToEdge(time))
+
+    def snapOutTime(self, time):
+        self.setOutTime(instance.PiTiVi.current.timeline.snapTimeToEdge(time))
+
     def _startDurationChangedCb(self, gnlobject, property):
         """ start/duration time has changed """
-        gst.log("self:%r , gnlobject:%r" % (self, gnlobject))
+        gst.log("self:%r , gnlobject:%r %r" % (self, gnlobject, self.gnlobject))
         if not gnlobject == self.gnlobject:
-            gst.warning("We're receiving signals from an object we dont' control (self.gnlobject:%r, gnlobject:%r)" % (self.gnlobject, gnlobject))
+            gst.warning("Receiving signals from object we don't control (ours:%r,sender:%r)"
+                % (self.gnlobject, gnlobject))
         self.gnlobject.debug("property:%s" % property.name)
         start = gst.CLOCK_TIME_NONE
         duration = 0
         if property.name == "start":
             start = gnlobject.get_property("start")
-            gst.log("start: %s => %s" % (gst.TIME_ARGS(self.start),
+            gst.log("start: %s => %s" % (gst.TIME_ARGS(self._start),
                                          gst.TIME_ARGS(start)))
-            if start == self.start:
+            if start == self._start:
                 start = gst.CLOCK_TIME_NONE
             else:
-                self.start = long(start)
+                self._start = long(start)
         elif property.name == "duration":
             duration = gnlobject.get_property("duration")
-            gst.log("duration: %s => %s" % (gst.TIME_ARGS(self.duration),
+            gst.log("duration: %s => %s" % (gst.TIME_ARGS(self._duration),
                                             gst.TIME_ARGS(duration)))
-            if duration == self.duration:
+            if duration == self._duration:
                 duration = 0
             else:
                 self.gnlobject.debug("duration changed:%s" % gst.TIME_ARGS(duration))
-                self.duration = long(duration)
-        self.emit("start-duration-changed", self.start, self.duration)
+                self._duration = long(duration)
+        self.emit("start-duration-changed", self._start, self._duration)
 
-
-    # Serializable methods
+## Serializable methods
 
     def toDataFormat(self):
         ret = BrotherObjects.toDataFormat(self)
         ret["start"] = self.start
-        ret["duration"] = self.duration
+        ret["duration"] = self._duration
         ret["name"] = self.name
-        if self.factory:
-            ret["factory-uid"] = self.factory.getUniqueID()
+        if self._factory:
+            ret["factory-uid"] = self._factory.getUniqueID()
         ret["media_type"] = self.media_type
         return ret
 
     def fromDataFormat(self, obj):
         BrotherObjects.fromDataFormat(self, obj)
-        self.start = obj["start"]
-        self.duration = obj["duration"]
+        self._start = obj["start"]
+        self._duration = obj["duration"]
 
         self.name = obj["name"]
 
@@ -434,3 +588,4 @@ class TimelineObject(BrotherObjects):
 
     def isVideo(self):
         return self.media_type == MEDIA_TYPE_VIDEO
+
