@@ -20,32 +20,17 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-import os.path
 import gobject
-import time
 import gtk
 import gst
-import gst.interfaces
-from glade import GladeWindow
 
-import plumber
+import pitivi.plumber as plumber
 import pitivi.instance as instance
 from pitivi.bin import SmartTimelineBin
-import pitivi.dnd as dnd
 from pitivi.signalgroup import SignalGroup
 
-from gettext import gettext as _
-
-def time_to_string(value):
-    if value == -1:
-        return "--:--:--.---"
-    ms = value / gst.MSECOND
-    sec = ms / 1000
-    ms = ms % 1000
-    mins = sec / 60
-    sec = sec % 60
-    hours = mins / 60
-    return "%02d:%02d:%02d.%03d" % (hours, mins, sec, ms)
+from pitivi.utils import time_to_string
+import dnd
 
 class PitiviViewer(gtk.VBox):
     """ Pitivi's viewer widget with controls """
@@ -54,7 +39,7 @@ class PitiviViewer(gtk.VBox):
         gst.log("New PitiviViewer")
         gtk.VBox.__init__(self)
         self.current_time = long(0)
-        self.requested_time = long(0)
+        self.requested_time = gst.CLOCK_TIME_NONE
         self.current_frame = -1
         self.valuechangedid = 0
         self.currentlySeeking = False
@@ -75,7 +60,7 @@ class PitiviViewer(gtk.VBox):
         # signal for timeline duration changes : (composition, sigid)
         self._timelineDurationChangedSigId = (None, None)
 
-        self._addTimelineToPlayground()
+
 
     def _connectToProject(self, project):
         """Connect signal handlers to a project.
@@ -88,6 +73,10 @@ class PitiviViewer(gtk.VBox):
                                      None, self._tmpIsReadyCb)
         self.project_signals.connect(project, "settings-changed",
                                      None, self._settingsChangedCb)
+        # we should add the timeline to the playground here, so
+        # that the new timeline bin will be added to the
+        # playground when the project loads
+        self._addTimelineToPlayground()
 
     def _createUi(self):
         """ Creates the Viewer GUI """
@@ -95,10 +84,11 @@ class PitiviViewer(gtk.VBox):
         self.set_spacing(5)
 
         # drawing area
-        self.aframe = gtk.AspectFrame(xalign=0.5, yalign=0.5, ratio=4.0/3.0, obey_child=False)
+        self.aframe = gtk.AspectFrame(xalign=0.5, yalign=0.5, ratio=4.0/3.0,
+                                      obey_child=False)
         self.pack_start(self.aframe, expand=True)
         self.drawingarea = ViewerWidget()
-        self.drawingarea.connect_after("realize", self._drawingAreaRealizeCb)
+        self.drawingarea.connect_after("expose-event", self._drawingAreaExposeCb)
         self.aframe.add(self.drawingarea)
 
         # Slider
@@ -145,11 +135,17 @@ class PitiviViewer(gtk.VBox):
 
         # current time
         self.timelabel = gtk.Label()
-        self.timelabel.set_markup("<tt>00m00s000 / --m--s---</tt>")
+        self.timelabel.set_markup("<tt>00:00:00.000 / --:--:--.---</tt>")
         self.timelabel.set_alignment(1.0, 0.5)
         self.timelabel.set_padding(5, 5)
         bbox.pack_start(self.timelabel, expand=False, padding=10)
 
+        # self.detach_button = gtk.Button()
+        # image = gtk.Image()
+        # image.set_from_stock(gtk.STOCK_LEAVE_FULLSCREEN,
+        #     gtk.ICON_SIZE_SMALL_TOOLBAR)
+        # self.detach_button.set_image(image)
+        # bbox.pack_end(self.detach_button, expand=False, fill=False)
 
         # drag and drop
         self.drag_dest_set(gtk.DEST_DEFAULT_DROP | gtk.DEST_DEFAULT_MOTION,
@@ -157,28 +153,14 @@ class PitiviViewer(gtk.VBox):
                            gtk.gdk.ACTION_COPY)
         self.connect("drag_data_received", self._dndDataReceivedCb)
 
-    def _asyncFrameRatioChange(self, ratio):
-        gst.debug("ratio:%f" % ratio)
-        self.aframe.set_property("ratio", ratio)
 
-    def _videosinkCapsNotifyCb(self, sinkpad, unused_property):
-        caps = sinkpad.get_negotiated_caps()
-        if not caps:
-            return
-        gst.log("caps:%s" % caps.to_string())
+    def setDisplayAspectRatio(self, ratio):
+        """ Sets the DAR of the Viewer to the given ratio """
+        gst.debug("Setting ratio of %f [%r]" % (float(ratio), ratio))
         try:
-            width = caps[0]["width"]
-            height = caps[0]["height"]
+            self.aframe.set_property("ratio", float(ratio))
         except:
-            gst.warning("Something went wrong when getting the video sink aspect ratio")
-        else:
-            try:
-                par = caps[0]["pixel-aspect-ratio"]
-            except:
-                # set aspect ratio
-                gobject.idle_add(self._asyncFrameRatioChange, float(width) / float(height))
-            else:
-                gobject.idle_add(self._asyncFrameRatioChange, float(width * par.num) / float(par.denom * height))
+            gst.warning("could not set ratio !")
 
     def _createSinkThreads(self):
         """ Creates the sink threads for the playground """
@@ -196,9 +178,6 @@ class PitiviViewer(gtk.VBox):
         vscale.link(vqueue)
         vsinkthread.videosink = self.videosink
         vsinkthread.add_pad(gst.GhostPad("sink", cspace.get_pad('sink')))
-
-        self.videosink.get_pad("sink").connect("notify::caps", self._videosinkCapsNotifyCb)
-
         self.drawingarea.videosink = self.videosink
 
         # audio elements
@@ -216,7 +195,8 @@ class PitiviViewer(gtk.VBox):
         # setting sinkthreads on playground
         instance.PiTiVi.playground.setVideoSinkThread(vsinkthread)
         instance.PiTiVi.playground.setAudioSinkThread(asinkthread)
-        instance.PiTiVi.playground.connect("current-changed", self._currentPlaygroundChangedCb)
+        instance.PiTiVi.playground.connect("current-changed",
+                                           self._currentPlaygroundChangedCb)
 
     def _settingsChangedCb(self, unused_project):
         gst.info("current project settings changed")
@@ -224,12 +204,15 @@ class PitiviViewer(gtk.VBox):
         # FIXME : do we really need to modify the ratio ??
         pass
 
-    def _drawingAreaRealizeCb(self, drawingarea):
+    def _drawingAreaExposeCb(self, drawingarea, event):
+        drawingarea.disconnect_by_func(self._drawingAreaExposeCb)
         drawingarea.modify_bg(gtk.STATE_NORMAL, drawingarea.style.black)
         self._createSinkThreads()
         if not instance.PiTiVi.playground.play() == gst.STATE_CHANGE_FAILURE:
             self.currentState = gst.STATE_PLAYING
 
+        return False
+
     ## gtk.HScale callbacks for self.slider
 
     def _sliderButtonPressCb(self, slider, unused_event):
@@ -280,24 +263,34 @@ class PitiviViewer(gtk.VBox):
             self._doSeek(seekvalue, gst.FORMAT_DEFAULT)
 
     def _seekTimeoutCb(self):
+        gst.debug("requested_time %s" % gst.TIME_ARGS(self.requested_time))
         self.currentlySeeking = False
-        if not self.current_time == self.requested_time:
+        if (self.requested_time != gst.CLOCK_TIME_NONE) and (self.current_time != self.requested_time):
             self._doSeek(self.requested_time)
+        return False
 
     def _doSeek(self, value, format=gst.FORMAT_TIME):
+        gst.debug("%s , currentlySeeking:%r" % (gst.TIME_ARGS(value),
+                                                self.currentlySeeking))
         if not self.currentlySeeking:
             self.currentlySeeking = True
-            gobject.timeout_add(80, self._seekTimeoutCb)
-            instance.PiTiVi.playground.seekInCurrent(value, format=format)
-            self._newTime(value)
-        if format == gst.FORMAT_TIME:
-            self.requested_time = value
+            if instance.PiTiVi.playground.seekInCurrent(value, format=format):
+                gst.debug("seek succeeded, request_time = NONE")
+                self.requested_time = gst.CLOCK_TIME_NONE
+                gobject.timeout_add(80, self._seekTimeoutCb)
+                self._newTime(value)
+            else:
+                self.currentlySeeking = False
+        else:
+            if format == gst.FORMAT_TIME:
+                self.requested_time = value
 
     def _newTime(self, value, frame=-1):
         gst.info("value:%s, frame:%d" % (gst.TIME_ARGS(value), frame))
         self.current_time = value
         self.current_frame = frame
-        self.timelabel.set_markup("<tt>%s / %s</tt>" % (time_to_string(value), time_to_string(instance.PiTiVi.playground.current.length)))
+        self.timelabel.set_markup("<tt>%s / %s</tt>" % (time_to_string(value),
+                                                        time_to_string(instance.PiTiVi.playground.current.length)))
         if not self.moving_slider:
             self.posadjust.set_value(float(value))
         return False
@@ -309,7 +302,8 @@ class PitiviViewer(gtk.VBox):
         gst.debug("duration : %s" % gst.TIME_ARGS(duration))
         gst.debug("playground.current.length : %s" % gst.TIME_ARGS(instance.PiTiVi.playground.current.length))
         self.posadjust.upper = float(duration)
-        self.timelabel.set_markup("<tt>%s / %s</tt>" % (time_to_string(self.current_time), time_to_string(instance.PiTiVi.playground.current.length)))
+        self.timelabel.set_markup("<tt>%s / %s</tt>" % (time_to_string(self.current_time),
+                                                        time_to_string(instance.PiTiVi.playground.current.length)))
 
 
     def _backToDefaultCb(self):
@@ -353,8 +347,14 @@ class PitiviViewer(gtk.VBox):
         self._connectToProject(project)
 
     def _addTimelineToPlayground(self):
-        instance.PiTiVi.playground.addPipeline(instance.PiTiVi.current.getBin())
-
+        # remove old timeline before proceeding
+        pg = instance.PiTiVi.playground
+        timeline = pg.getTimeline()
+        if timeline:
+            pg.switchToDefault()
+            pg.removePipeline(timeline)
+        # add current timeline
+        pg.addPipeline(instance.PiTiVi.current.getBin())
 
     ## Control gtk.Button callbacks
 
@@ -378,7 +378,6 @@ class PitiviViewer(gtk.VBox):
     def _forwardCb(self, unused_button):
         pass
 
-
     ## Playground callbacks
 
     def _playgroundPositionCb(self, unused_playground, unused_smartbin, pos):
@@ -386,7 +385,9 @@ class PitiviViewer(gtk.VBox):
 
     def _currentPlaygroundChangedCb(self, playground, smartbin):
         gst.log("smartbin:%s" % smartbin)
-        if smartbin == playground.default:
+
+        if not smartbin.seekable:
+            # live sources or defaults, no duration/seeking available
             self.slider.set_sensitive(False)
             self.playpause_button.set_sensitive(False)
             self.next_button.set_sensitive(False)
@@ -397,7 +398,8 @@ class PitiviViewer(gtk.VBox):
                 self._timelineDurationChangedSigId = (None, None)
         else:
             if isinstance(smartbin, SmartTimelineBin):
-                gst.info("switching to Timeline, setting duration to %s" % (gst.TIME_ARGS(smartbin.project.timeline.videocomp.duration)))
+                gst.info("switching to Timeline, setting duration to %s" %
+                         (gst.TIME_ARGS(smartbin.project.timeline.videocomp.duration)))
                 self.posadjust.upper = float(smartbin.project.timeline.videocomp.duration)
                 # FIXME : we need to disconnect from this signal !
                 sigid = smartbin.project.timeline.videocomp.connect("start-duration-changed",
@@ -405,7 +407,7 @@ class PitiviViewer(gtk.VBox):
                 self._timelineDurationChangedSigId = (smartbin.project.timeline.videocomp,
                                                       sigid)
             else:
-                self.posadjust.upper = float(smartbin.factory.length)
+                self.posadjust.upper = float(smartbin.factory.duration)
                 if not self._timelineDurationChangedSigId == (None, None):
                     obj, sigid = self._timelineDurationChangedSigId
                     obj.disconnect(sigid)
@@ -416,6 +418,15 @@ class PitiviViewer(gtk.VBox):
             self.next_button.set_sensitive(True)
             self.back_button.set_sensitive(True)
 
+        if isinstance(smartbin, SmartTimelineBin):
+            seti = smartbin.project.getSettings()
+            dar = float(seti.videowidth * seti.videopar.num) / float(seti.videoheight * seti.videopar.denom)
+        elif hasattr(smartbin, 'factory'):
+            dar = smartbin.factory.video_info_stream.dar
+        else:
+            dar = smartbin.width / smartbin.height
+        self.setDisplayAspectRatio(dar)
+
     def _currentStateCb(self, unused_playground, state):
         gst.info("current state changed : %s" % state)
         if state == int(gst.STATE_PLAYING):
@@ -430,9 +441,7 @@ class PitiviViewer(gtk.VBox):
             dav = self.drawingarea.videosink
             gst.log('%s' % dav)
             if dav and dav.realsink and dav.realsink == message.src:
-                self.drawingarea.can_set_xid = True
-                if self.drawingarea.isexposed:
-                    self.drawingarea.set_xwindow_id()
+                self.drawingarea.set_xwindow_id()
 
 
 class ViewerWidget(gtk.DrawingArea):
@@ -446,21 +455,8 @@ class ViewerWidget(gtk.DrawingArea):
         gtk.DrawingArea.__init__(self)
         self.videosink = None
         self.have_set_xid = False
-        self.can_set_xid = False
         self.unset_flags(gtk.DOUBLE_BUFFERED)
         self.unset_flags(gtk.SENSITIVE)
-        self.isexposed = False
-
-    def do_expose_event(self, unused_event):
-        """ 'expose-event' override """
-        gst.log("expose have_set_xid:%d can_set_xid:%d" % (self.have_set_xid, self.can_set_xid))
-        self.isexposed = True
-        if self.videosink:
-            if not self.have_set_xid and self.can_set_xid:
-                self.set_xwindow_id()
-            elif self.have_set_xid:
-                self.videosink.expose()
-        return False
 
     def set_xwindow_id(self):
         """ set the widget's XID on the configured videosink. """
@@ -512,5 +508,3 @@ class PlayPauseButton(gtk.Button):
             self.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_BUTTON))
             self.playing = True
 
-
-
